Package: RcppDynProg
Type: Package
Title: 'Rcpp' Dynamic Programming
Version: 0.2.1
Date: 2023-08-19
Authors@R: c(
    person("John", "Mount", email = "jmount@win-vector.com", role = c("aut", "cre")),
    person("Nina", "Zumel", email = "nzumel@win-vector.com", role = c("aut")),
    person(family = "Win-Vector LLC", role = c("cph"))
    )
URL: https://github.com/WinVector/RcppDynProg/,
        https://winvector.github.io/RcppDynProg/
BugReports: https://github.com/WinVector/RcppDynProg/issues
Maintainer: John Mount <jmount@win-vector.com>
Description: Dynamic Programming implemented in 'Rcpp'.  Includes example partition and out of sample fitting applications.  Also supplies additional custom coders for the 'vtreat' package.
License: GPL-2 | GPL-3
Depends: R (>= 3.4.0)
Imports: wrapr (>= 2.0.4), Rcpp (>= 1.0.0), utils, stats
LinkingTo: Rcpp, RcppArmadillo
RoxygenNote: 7.2.3
Suggests: tinytest, knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2023-08-20 00:28:18 UTC; johnmount
Author: John Mount [aut, cre],
  Nina Zumel [aut],
  Win-Vector LLC [cph]
Repository: CRAN
Date/Publication: 2023-08-20 02:32:36 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 21:37:46 UTC; unix
Archs: RcppDynProg.so.dSYM
