# Generated by roxygen2: do not edit by hand

export(all_partitions)
export(const_cost)
export(const_cost_logistic)
export(const_costs)
export(const_costs_logistic)
export(lin_cost)
export(lin_cost_logistic)
export(lin_costs)
export(lin_costs_logistic)
export(logistic_fits)
export(logistic_solve1)
export(piecewise_constant)
export(piecewise_constant_coder)
export(piecewise_linear)
export(piecewise_linear_coder)
export(score_solution)
export(solve_for_partition)
export(solve_for_partitionc)
export(solve_interval_partition)
export(solve_interval_partition_R)
export(solve_interval_partition_k)
export(solve_interval_partition_no_k)
export(summarize_input)
export(xlin_fits)
export(xlin_fits_R)
export(xlin_fits_V)
export(xlin_fits_lm)
export(xlin_pfits)
export(xlogistic_fits)
import(Rcpp)
importFrom(Rcpp,evalCpp)
importFrom(stats,approx)
importFrom(stats,lm)
importFrom(stats,predict)
importFrom(utils,combn)
importFrom(wrapr,stop_if_dot_args)
useDynLib(RcppDynProg)
