---
title: "RcppDynProg package"
author: "John Mount"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{RcppDynProg package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

This content has moved to [the package README](https://github.com/WinVector/RcppDynProg).